@@ -10,6 +10,7 @@ module Yesod.Core
     , breadcrumbs
       -- * Types
     , Approot (..)
+    , FileUpload (..)
       -- * Utitlities
     , maybeAuthorized
     , widgetToPageContent
@@ -20,17 +21,22 @@ module Yesod.Core
     , unauthorizedI
       -- * Logging
     , LogLevel (..)
-    , formatLogMessage
-    , fileLocationToString
     , logDebug
     , logInfo
     , logWarn
     , logError
     , logOther
+    , logDebugS
+    , logInfoS
+    , logWarnS
+    , logErrorS
+    , logOtherS
       -- * Sessions
     , SessionBackend (..)
     , defaultClientSessionBackend
     , clientSessionBackend
+    , clientSessionBackend2
+    , clientSessionDateCacher
     , loadClientSession
     , Header(..)
     , BackendSession
@@ -41,6 +47,7 @@ module Yesod.Core
       -- * Misc
     , yesodVersion
     , yesodRender
+    , runFakeHandler
       -- * Re-exports
     , module Yesod.Content
     , module Yesod.Dispatch
@@ -59,38 +66,7 @@ import Yesod.Request
 import Yesod.Widget
 import Yesod.Message
 
-import Language.Haskell.TH.Syntax
-import qualified Language.Haskell.TH.Syntax as TH
-import Data.Text (Text)
-
-logTH :: LogLevel -> Q Exp
-logTH level =
-    [|messageLoggerHandler $(qLocation >>= liftLoc) $(TH.lift level)|]
-  where
-    liftLoc :: Loc -> Q Exp
-    liftLoc (Loc a b c d e) = [|Loc $(TH.lift a) $(TH.lift b) $(TH.lift c) $(TH.lift d) $(TH.lift e)|]
-
--- | Generates a function that takes a 'Text' and logs a 'LevelDebug' message. Usage:
---
--- > $(logDebug) "This is a debug log message"
-logDebug :: Q Exp
-logDebug = logTH LevelDebug
-
--- | See 'logDebug'
-logInfo :: Q Exp
-logInfo = logTH LevelInfo
--- | See 'logDebug'
-logWarn :: Q Exp
-logWarn = logTH LevelWarn
--- | See 'logDebug'
-logError :: Q Exp
-logError = logTH LevelError
-
--- | Generates a function that takes a 'Text' and logs a 'LevelOther' message. Usage:
---
--- > $(logOther "My new level") "This is a log message"
-logOther :: Text -> Q Exp
-logOther = logTH . LevelOther
+import Control.Monad.Logger
 
 -- | Return an 'Unauthorized' value, with the given i18n message.
 unauthorizedI :: RenderMessage master msg => msg -> GHandler sub master AuthResult
