@@ -12,21 +12,22 @@ import YesodCoreTest.Cache
 import qualified YesodCoreTest.WaiSubsite as WaiSubsite
 import qualified YesodCoreTest.Redirect as Redirect
 import qualified YesodCoreTest.JsLoader as JsLoader
+import qualified YesodCoreTest.RequestBodySize as RequestBodySize
 
 import Test.Hspec
 
-specs :: [Spec]
-specs = 
-    [ cleanPathTest
-    , exceptionsTest
-    , widgetTest
-    , mediaTest
-    , linksTest
-    , noOverloadedTest
-    , internalRequestTest
-    , errorHandlingTest
-    , cacheTest
-    , WaiSubsite.specs
-    , Redirect.specs
-    , JsLoader.specs
-    ]
+specs :: Spec
+specs = do
+      cleanPathTest
+      exceptionsTest
+      widgetTest
+      mediaTest
+      linksTest
+      noOverloadedTest
+      internalRequestTest
+      errorHandlingTest
+      cacheTest
+      WaiSubsite.specs
+      Redirect.specs
+      JsLoader.specs
+      RequestBodySize.specs
