@@ -4,7 +4,6 @@
 module YesodCoreTest.Cache (cacheTest, Widget) where
 
 import Test.Hspec
-import Test.Hspec.HUnit()
 
 import Network.Wai
 import Network.Wai.Test
@@ -37,9 +36,8 @@ getRootR = do
 
 cacheTest :: Spec
 cacheTest =
-  describe "Test.Cache"
-    [ it "works" works
-    ]
+  describe "Test.Cache" $ do
+      it "works" works
 
 runner :: Session () -> IO ()
 runner f = toWaiApp C >>= runSession f
