@@ -4,7 +4,6 @@
 module YesodCoreTest.Exceptions (exceptionsTest, Widget) where
 
 import Test.Hspec
-import Test.Hspec.HUnit ()
 
 import Yesod.Core hiding (Request)
 import Network.Wai
@@ -31,10 +30,9 @@ getRedirR = do
     redirectWith status301 RootR
 
 exceptionsTest :: Spec
-exceptionsTest = describe "Test.Exceptions"
-    [ it "500" case500
-    , it "redirect keeps headers" caseRedirect
-    ]
+exceptionsTest = describe "Test.Exceptions" $ do
+      it "500" case500
+      it "redirect keeps headers" caseRedirect
 
 runner :: Session () -> IO ()
 runner f = toWaiApp Y >>= runSession f
