@@ -3,7 +3,6 @@
 module YesodCoreTest.NoOverloadedStrings (noOverloadedTest, Widget) where
 
 import Test.Hspec
-import Test.Hspec.HUnit ()
 
 import Yesod.Core hiding (Request)
 import Network.Wai.Test
@@ -45,6 +44,5 @@ case_sanity = runner $ do
     assertBody mempty res
 
 noOverloadedTest :: Spec
-noOverloadedTest = describe "Test.NoOverloadedStrings"
-    [ it "sanity" case_sanity
-    ]
+noOverloadedTest = describe "Test.NoOverloadedStrings" $ do
+      it "sanity" case_sanity
